<?php

    /**
    * Project : Technical Project Manager (IEEE like)
    *
    * This screen show tasks plan by assignee. Unassigned tasks are shown 
    * below assigned tasks
    *
    * @package mod-techproject
    * @category mod
    * @author Valery Fremaux (France) (admin@www.ethnoinformatique.fr)
    * @date 2008/03/03
    * @version phase1
    * @contributors LUU Tao Meng, So Gerard (parts of treelib.php), Guillaume Magnien, Olivier Petit
    * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
    */


    $TIMEUNITS = array(get_string('unset','techproject'),get_string('hours','techproject'),get_string('halfdays','techproject'),get_string('days','techproject'));
    $haveAssignedTasks = false;
    
    if (!groupmode($project->course, $cm)){
        $groupusers = get_course_users($project->course);
    } else {
        if ($currentGroupId){
            $groupusers = get_group_users($currentGroupId);
        } else {
            // we could not rely on the legacy function
            $groupusers = techproject_get_users_not_in_group($project->course);
        }
    }
    
    if (!isset($groupusers) || count($groupusers) == 0 || empty($groupusers)){
        print_simple_box(get_string('noassignee','techproject'), 'center');
    } else {
        print_heading_block(get_string('assignedtasks','techproject'));
        echo '<br/>';
        print_simple_box_start('center', '100%');
        foreach($groupusers as $aUser){
    ?>
    <table width="100%">
        <tr>
            <td class="byassigneeheading level1">
    <?php 
        	$hidesub = "<a href=\"javascript:toggle('{$aUser->id}','sub{$aUser->id}');\"><img name=\"img{$aUser->id}\" src=\"{$CFG->pixpath}/p/switch_minus.gif\" alt=\"collapse\" style=\"background-color : #E0E0E0\" /></a>";
            echo $hidesub.' '.get_string('assignedto','techproject').' '.fullname($aUser).' '.print_user_picture ($aUser->id, $project->course, !empty($aUser->image), 0, true, true ); 
    ?>
            </td>
            <td class="byassigneeheading level1" align="right">
    <?php
            $query = "
               SELECT 
                  SUM(planned) as planned,
                  SUM(done) as done,
                  SUM(cost) as cost,
                  COUNT(*) as count
               FROM
                  {$CFG->prefix}techproject_task as t
               WHERE
                  t.projectId = {$project->id} AND
                  t.groupId = {$currentGroupId} AND
                  t.assignee = {$aUser->id}
               GROUP BY
                  t.assignee
            ";
            $res = get_record_sql($query);
            $over = ($res->planned) ? round((($res->cost - $res->planned) / $res->planned) * 100) : 0 ;
            // calculates a local alarm for lateness
            $hurryup = '';
            if ($res->planned && ($res->cost <= $res->planned)){
                $hurryup = (round(($res->cost / $res->planned) * 100) > ($res->done / $res->count)) ? "<img src=\"{$CFG->pixpath}/p/late.gif\" title=\"".mb_convert_encoding(get_string('hurryup','techproject'), 'UTF8', 'ISO-8859-1')."\" />" : '' ;
            }
            $lateclass = ($over > 0) ? 'toolate' : 'intime';
            $workplan = get_string('assignedwork','techproject').' '.(0 + $res->planned).' '.$TIMEUNITS[$project->timeunit];
            $realwork = get_string('realwork','techproject')." <span class=\"{$lateclass}\">".(0 + $res->cost).' '.$TIMEUNITS[$project->timeunit].'</span>';
    	    $completion = ($res->count != 0) ? techproject_bar_graph_over($res->done / $res->count, $over, 100, 10) : techproject_bar_graph_over(-1, 0);
            echo "{$workplan} - {$realwork} {$completion} {$hurryup}";
    ?>        
            </td>
        </tr>
    </table>
    
    <table id="<?php echo "sub{$aUser->id}" ?>" width="100%">
    <?php
        
            // get assigned tasks
            $query = "
               SELECT
                  t.*,
                  ts.label as statuslabel,
                  COUNT(tts.specId) as specs
               FROM
                  {$CFG->prefix}techproject_taskstatus as ts,
                  {$CFG->prefix}techproject_task as t
               LEFT JOIN
                  {$CFG->prefix}techproject_task_to_spec as tts
               ON
                  tts.taskId = t.id
               WHERE
                  t.projectId = {$project->id} AND
                  t.groupId = {$currentGroupId} AND
                  ts.taskstatus = t.status AND
                  t.assignee = {$aUser->id}
               GROUP BY
                  t.id
            ";
            $tasks = get_records_sql($query);
            if (!isset($tasks) || count($tasks) == 0 || empty($tasks)){
    ?>
        <tr>
            <td>
                <?php print_string('notaskassigned', 'techproject') ?>
            </td>
        </tr>
    <?php        
            } else {
                foreach($tasks as $aTask){
                    $haveAssignedTasks = true;
                    // feed milestone titles for popup display
                    if ($milestone = get_record('techproject_milestone', 'id', $aTask->milestoneid)){
                        $aTask->milestoneabstract = $milestone->abstract;
                    }
    ?>
        <tr>
            <td class="level2">
            <?php techproject_print_single_task($aTask, $project, $currentGroupId, $cm->id, count($tasks), true, 'SHORT_WITHOUT_ASSIGNEE_NOEDIT'); ?>
            </td>
        </tr>
    <?php
                }
            }
    ?>
    </table>
    <?php
        }
        print_simple_box_end();
    }
    
    // get unassigned tasks
    $query = "
       SELECT
          *
       FROM
          {$CFG->prefix}techproject_task
       WHERE
          projectId = {$project->id} AND
          groupId = {$currentGroupId} AND
          assignee = 0
    ";
    $unassignedtasks = get_records_sql($query);
    print_heading_block(get_string('unassignedtasks','techproject'));
    ?>
    <br/>
    <?php
    print_simple_box_start('center', '100%');
    ?>
    <center>
    <table width="100%">
    <?php
    if (!isset($unassignedtasks) || count($unassignedtasks) == 0 || empty($unassignedtasks)){
    ?>
        <tr>
            <td>
                <?php print_string('notaskunassigned', 'techproject') ?>
            </td>
        </tr>
    <?php        
    } else {
        foreach($unassignedtasks as $aTask){
    ?>
        <tr>
            <td class="level2">
                <?php
                $branch = techproject_tree_get_upper_branch('techproject_task', $aTask->id, true, true);
                echo 'T'.implode('.', $branch) . '. ' . $aTask->abstract ;
                echo "&nbsp;<a href=\"view.php?id={$cm->id}&amp;view=view_detail&amp;objectClass=task&amp;objectId=$aTask->id\"><img src=\"{$CFG->pixpath}/p/hide.gif\" title=\"".get_string('detail','techproject')."\" /></a>";
                ?>
            </td>
            <td>
            </td>
        </tr>
    <?php
        }
    }
    ?>
    </table>
    </center>
<?php 
    print_simple_box_end();
?>