<?php  // $Id: treelib.php,v 1.4.4.3 2008/05/26 20:59:19 diml Exp $

/**
* Project : Technical Project Manager (IEEE like)
*
* A special lib for handling tree-shaped entities.
*
* @package mod-techproject
* @category mod
* @author So Gerard (EISTI 2002)
* @date 2008/03/03
* @version phase 1
* @contributors Valery Fremaux (France) (admin@www.ethnoinformatique.fr)
* @license http://www.gnu.org/copyleft/gpl.html GNU Public License
*/

/// Library of tree dedicated operations

/*** Index of functions **********************
techproject_tree_delete($id, $table);
techproject_tree_updateordering($id, $table, $istree);
techproject_tree_up($id, $table);
techproject_tree_down($id, $table);
techproject_tree_left($id, $table);
techproject_tree_right($id, $table);
techproject_get_tree_options($table, $projectid, $groupid, $fatherid = 0, $ordering = '');
techproject_get_subtree_list($table, $id);
techproject_count_subs($table, $id)
techproject_count_leaves($table, $id, $returnList=false)
techproject_tree_propagate_up($table, $field, $id, $function='+', $byFather = false);
techproject_tree_get_upper_branch($table, $id, $includeStart = false, $returnordering = false)
techproject_tree_copy_set($set, $fromtable, $totable, $copy = false);
techproject_tree_get_max_ordering($project, $group, $table, $istree = false, $fatherid = 0)
techproject_tree_get_tree($table, $projectid, $groupid)
**********************************************/

/**
* deletes into tree a full branch. note that it will work either
* @param id the root node id
* @param table the table where the tree is in 
* @param istree if istree is not set, considers table as a simple ordered list
* @return an array of deleted ids
*/
function techproject_tree_delete($id, $table, $istree = 1){
		techproject_tree_updateordering($id, $table, $istree);
		return tree_delete_rec($id, $table, $istree);
}

/**
* deletes recursively a node and its subnodes. this is the recursion deletion
* @return an array of deleted ids
*/
function tree_delete_rec($id, $table, $istree){
	global $CFG;

    $deleted = array();
    if (empty($id)) return $deleted;    

	// echo "deleting $id<br/>";
	
	// getting all subnodes to delete if is tree.
	if ($istree){
    	$query = "
    	    SELECT 
    	        id
    	    FROM 
    	        {$CFG->prefix}{$table}
    	    WHERE
    	        fatherid = {$id}
    	";
    
    	// deleting subnodes if any
    	if ($subs = get_record_sql($query)) {
    		foreach($subs as $aSub){
    			$deleted = array_merge($deleted, tree_delete_rec($aSub->id, $table, $istree));
    		}
    	}
    }
	// deleting current node
	delete_records($table, 'id', $id); 
	$deleted[] = $id;
	return $deleted;
}

/**
* updates ordering of a tree branch from a specific node, reordering 
* all subsequent siblings. 
* @param id the node from where to reorder
* @param table the table-tree
*/
function techproject_tree_updateordering($id, $table, $istree){

	// getting ordering value of the current node
	global $CFG;

	$res =  get_record($table, 'id', $id);
	if (!$res) return;

	$treeclause = ($istree) ? "     AND fatherid = {$res->fatherid} " : '';

	// getting subsequent nodes that have same father
	$query = "
	    SELECT 
	        id   
	    FROM 
	        {$CFG->prefix}$table
	    WHERE 
	        ordering > {$res->ordering}
	        $treeclause
	    ORDER BY 
	        ordering
	";

	// reordering subsequent nodes using an object
	if( $nextsubs = get_record_sql($query)) {
	    $ordering = $res->ordering + 1;
		foreach($nextsubs as $asub){
			$objet->id = $asub->id;
			$objet->ordering = $ordering;
			update_record($table, $objet);
			$ordering++;
		}
	}
}

/**
* raises a node in the tree, reordering all what needed
* @param id the id of the raised node
* @param table the table-tree where to operate
* @param istree true if is a table-tree rather than a table-list
* @return void
*/
function techproject_tree_up($project, $group, $id, $table, $istree = 1){
	global $CFG;

	$res =  get_record($table, 'id', $id);
	if (!$res) return;

	$treeclause = ($istree) ? "     AND fatherid = {$res->fatherid} " : '';

	if($res->ordering > 1){
		$newordering = $res->ordering - 1;

		$query = "
		    SELECT 
		        id
		    FROM 
		        {$CFG->prefix}$table
		    WHERE 
		        groupId = {$group} AND
		        projectId = {$project->id} AND
		        ordering = $newordering 
		        $treeclause
		";
		
		// echo $query;
		
		$result =  get_record_sql($query);
		$resid = $result->id;

        // swapping
		$objet->id = $resid;
		$objet->ordering = $res->ordering;
		update_record($table, $objet);

		$objet->id = $id;
		$objet->ordering = $newordering;
		update_record($table, $objet);
	}
}

/**
* lowers a node on its branch. this is done by swapping ordering.
* @param project the current project
* @param group the current group
* @param id the node id
* @param table the table-tree where to perform swap
* @param istree if not set, performs swapping on a single list
*/
function techproject_tree_down($project, $group, $id, $table, $istree = 1){
	global $CFG;

	$res =  get_record($table, 'id', $id);
	$treeclause = ($istree) ? "     AND fatherid = {$res->fatherid} " : '';

	$query = "
	    SELECT 
	        MAX(ordering) AS ordering
	    FROM 
	        {$CFG->prefix}$table
	    WHERE
	        projectId = {$project->id} AND
	        groupId = {$group}
	        $treeclause
	";
	
	// echo $query;

	$resmaxordering = get_record_sql($query);
	$maxordering = $resmaxordering->ordering;

	if($res->ordering < $maxordering){
		$newordering = $res->ordering + 1;

		$query = "
		    SELECT 
		        id
		    FROM    
		        {$CFG->prefix}$table
		    WHERE 
    	        projectId = {$project->id} AND
    	        groupId = {$group} AND
		        ordering = $newordering 
		        $treeclause
		";
		$result =  get_record_sql($query);
		$resid = $result->id;

        // swapping
		$objet->id = $resid;
		$objet->ordering = $res->ordering;
		update_record("$table", $objet);

		$objet->id = $id;
		$objet->ordering = $newordering;
		update_record("$table", $objet);

	}
}

/**
* raises a node to the upper level. Subsequent nodes become sons of the raised node
* @param project the current project
* @param group the current group
* @param id the node to be raised
* @param table the table-tree name
*/
function techproject_tree_left($project, $group, $id, $table){
	global $CFG;

	$query = "
	    SELECT 
	        fatherid, 
	        ordering
	    FROM 
	        {$CFG->prefix}$table
	    WHERE 
	        id = $id
	";
	$res =  get_record_sql($query);
	$ordering = $res->ordering;
	$fatherid = $res->fatherid;

	$query = "
	    SELECT 
	        id,
	        fatherid
	    FROM 
	        {$CFG->prefix}$table
	    WHERE 
	        id = $fatherid
	";
	$resfatherid =  get_record_sql($query);
	$fatheridbis = $resfatherid->fatherid; //id grand pere

	$query = "
	    SELECT 
	        id,
	        ordering
	    FROM 
	        {$CFG->prefix}$table
        WHERE 
	        projectId = {$project->id} AND
	        groupId = {$group} AND
            ordering > $ordering AND 
            fatherid = $fatherid
        ORDER BY 
            ordering
    ";
	$newbrotherordering = $ordering;

	if($ress = get_records_sql($query)){
		foreach($ress as $res){
			$objet->id = $res->id;
			$objet->ordering = $newbrotherordering;
			update_record("$table", $objet);
			$newbrotherordering = $newbrotherordering + 1;
		}
	}

	//On recupere l'ordering  du pere
	$query = "
	    SELECT
	        id, 
	        ordering
	    FROM 
	        {$CFG->prefix}$table
	    WHERE 
	        projectId = {$project->id} AND
	        groupId = {$group} AND
	        id = $fatherid
	";
	$resorderingfather =  get_record_sql($query);
	$orderingfather = $resorderingfather->ordering;

	//On decale la orderingition des freres du pere qui le suit
	$query = "
	    SELECT 
	        id,
	        ordering
	    FROM 
	        {$CFG->prefix}$table
	    WHERE 
	        projectId = {$project->id} AND
	        groupId = {$group} AND
	        ordering > {$orderingfather} AND 
	        fatherid = {$fatheridbis}
	    ORDER BY 
	        ordering
	";
	if ($resbrotherfathers = get_records_sql($query)) {
		foreach($resbrotherfathers as $resbrotherfather){
			$idbrotherfather = $resbrotherfather->id;
			$nextordering = $resbrotherfather->ordering + 1;

			$objet->id=$idbrotherfather;
			$objet->ordering = $nextordering;
			update_record("$table", $objet);
		}
	}

	//on insere la newordering
	$newordering = $orderingfather + 1;

	$objet->id = $id;
	$objet->ordering = $newordering;
	$objet->fatherid = $fatheridbis;
	update_record("$table", $objet);
}

/**
* lowers a node within its own branch setting it as 
* sub node of the previous sibling. The first son cannot be lowered.
* @param project the current project
* @param group the current group
* @param id the node to be lowered
* @param table the table-tree name
*/
function techproject_tree_right($project, $group, $id, $table){
	global $CFG;

	$query = "
	    SELECT 
	        fatherid, 
	        ordering,
	        projectId,
	        groupId
	    FROM 
	        {$CFG->prefix}$table
	    WHERE 
	        id = $id
	";
	$res =  get_record_sql($query);
	$ordering = $res->ordering;
	$fatherid = $res->fatherid;
	$group = $res->groupId;

	if( 1 < $ordering ){
		$orderingbis = $ordering - 1;

		$query = "
		    SELECT 
		        id,ordering
		    FROM 
		        {$CFG->prefix}$table
    		WHERE 
    	        projectId = {$project->id} AND
    	        groupId = {$group} AND
    		    ordering = $orderingbis AND 
    		    fatherid = $fatherid
        ";
		$resid = get_record_sql($query);
		$newfatherid = $resid->id;

		$query = "
		    SELECT 
		        id, 
		        ordering
		    FROM 
		        {$CFG->prefix}$table
		    WHERE 
    	        projectId = {$project->id} AND
    	        groupId = {$group} AND
		        ordering > $ordering AND 
		        fatherid = $fatherid 
		    ORDER BY 
		        ordering
		";
		$newbrotherordering = $ordering;

		if ($resbrothers = get_records_sql($query)) {
			foreach($resbrothers as $resbrother){
				$objet->id = $resbrother->id;
				$objet->ordering = $newbrotherordering;
				update_record("$table", $objet);
				$newbrotherordering = $newbrotherordering + 1;
			}
		}

		$maxordering = techproject_tree_get_max_ordering($project->id, $group, $table, true, $newfatherid);
		$newordering = $maxordering + 1;

		//assigning father's id
		$objet->id = $id;
		$objet->fatherid = $newfatherid;
		$objet->ordering = $newordering;
		update_record("$table", $objet);
	}
}

/**
* gets a full project tree for selection
* @param table the table-tree name
* @param projectid the current project module
* @param groupid the currently working group
* @param fatherid the father node in the tree
* @param ordering the ordering prefix for accumulating full ordering string.
* @return an ordered array of elements
*/
function techproject_get_tree_options($table, $projectid, $groupid, $fatherid = 0, $ordering = ''){
    global $CFG;


    $query = "
       SELECT
          id,
          ordering,
          abstract
       FROM
          {$CFG->prefix}{$table}
       WHERE
          projectid = {$projectid} AND
          groupid = {$groupid} AND
          fatherid = {$fatherid}
       ORDER BY
          ordering
    ";
    
    // echo $query;

    $collected = array();
    if ($elements = get_records_sql($query)){
        foreach($elements as $anElement){
            $anElement->ordering = (empty($ordering)) ? $anElement->ordering : $ordering . '.' . $anElement->ordering;
            $collected[] = $anElement;
            $collected = array_merge($collected, techproject_get_tree_options($table, $projectid, $groupid, $anElement->id, $anElement->ordering));
        }
    }
    
    return $collected;
}

/**
* get the full list of dependencies in a tree
* @param table the table-tree
* @param id the node from where to start of
* @return a comma separated list of nodes
*/
function techproject_get_subtree_list($table, $id){
    $res = get_records_menu($table, 'fatherid', $id);
    $ids = array();
    if (is_array($res)){
        foreach(array_keys($res) as $aSub){
            $ids[] = $aSub;
            $subs = techproject_get_subtree_list($table, $aSub);
            if (!empty($subs)) $ids[] = $subs;
        }
    }
    return(implode(',', $ids));
}

/**
* count direct subs in a tree
* @param table the table-tree
* @param the node
* @return the number of direct subs
*/
function techproject_count_subs($table, $id){
    global $CFG;
    
    // counting direct subs
	$query = "
	    SELECT 
	        COUNT(id) AS nbsub
	    FROM 
	        {$CFG->prefix}{$table}
	    WHERE 
	        fatherid = {$id}
	";
	$res = get_record_sql($query);
	return $res->nbsub;
}

/**
* count all items that are leaves in tree (effective entries)
* @param table the table-tree
* @param the node
* @param returnList if true, returns a list of leave's Ids, if false, returns the leaf count
* @return the number of leaf subs, or a list of leaves
*/
function techproject_count_leaves($table, $id, $returnList=false){
    global $CFG;
    
    if (techproject_count_subs($table, $id) == 0){
        if ($id == 0) ($returnList) ? array() : 0 ;
        return ($returnList) ? array($id) : 1 ;
    }
    
    $leaves = 0;
    $leafIds = array();
    
    // counting for direct subs
	$query = "
	    SELECT 
	        id,
	        abstract
	    FROM 
	        {$CFG->prefix}{$table}
	    WHERE 
	        fatherid = {$id}
	";
	$res = get_records_sql($query);
	
	if ($res){
        foreach($res as $aSub){
            if ($returnList){
                $leafIds = array_merge($leafIds, techproject_count_leaves($table, $aSub->id, true));
            }
            else{
                $leaves += techproject_count_leaves($table, $aSub->id, false);
            }
        }
    }
    	
	return ($returnList) ? $leafIds : $leaves ;
}

/**
* propagates a calculation in the tree up to a root node. defaults to mathematic meaning
* @param table the table-tree
* @param field the field that has concerns in the calculation
* @param id the id from where to propagate
* @param function the calculation
* @param byFather if true, the id given is the father node's id that from where the propagation is required. usefull
* when propagating after a record is deleted
*/
function techproject_tree_propagate_up($table, $field, $id, $function = '~', $byFather = false){
    global $CFG;
    
    if (!$byFather){
        $aNode = get_record($table, 'id', $id);
        $fatherid = $aNode->fatherid;
    }
    else{
       $fatherid = $id;
    }
    if ($fatherid){
        // get all brothers in this tree branch (including me)
        if($res = get_records_menu($table, 'fatherid', $fatherid, 'id', "id,$field")){
            // calculate mathematic meaning
            switch($function){
                case '~': {
                    $fieldValue = round(array_sum(array_values($res)) / count(array_keys($res)));
                } break;
                case '+':{
                    $fieldValue = round(array_sum(array_values($res)));
                } break;
            }
            
            // make a "father object"
            $theFather->id = $fatherid;
            $theFather->{$field} = $fieldValue;
            update_record($table, $theFather);
        }

        // continue propagation        
        techproject_tree_propagate_up($table, $field, $fatherid, $function);
    }
}

/**
* get upper branch to a node from root to node
* @param the table-tree where to oper
* @param id the node id to reach
* @param includeStart true if leaf node is in the list
* @return array of node ids
*/
function techproject_tree_get_upper_branch($table, $id, $includeStart = false, $returnordering = false){
    global $CFG;

    $nodelist = array();
    $res = get_record($table, 'id', $id);
    if ($includeStart) $nodelist[] = ($returnordering) ? $res->ordering : $id ;    
    while($res->fatherid != 0){
        $res = get_record($table, 'id', $res->fatherid);
        $nodelist[] = ($returnordering) ? $res->ordering : $res->id;
    }
    $nodelist = array_reverse($nodelist);
    return $nodelist;
}

/**
* copies or moves items from a table-tree to another, respecting structure.
* only standard descriptive attributes are copied (abstract and description), unless
* field parameter is used. Move is destructive (loss of data is possible). Copy
* is non destructive.
* Associations are deleted when moved, are not reported in copy. 
* @param array $set the set of ids to be copied/moved
* @param string $fromtable the origin table-tree given by name
* @param string $totable the destination table-tree given by name
* @param string $fields a list of comma separated fields to report
*/
function techproject_tree_copy_set($set, $fromtable, $totable, $fields = 'description,format,abstract,projectid,groupid,ordering'){
    global $CFG, $USER;

    // nothing to do
    if (count($set) == 0) return;
    
    // stores extracted objects
    $items = array();
    
    // links the clones within a tree topology from leaves, knowing the original branch sequence
    function place_in_tree($table, &$items, &$item){
        $branch = techproject_tree_get_upper_branch($table, $item->id);
        
        // for each node climbing upper the branch, do we find a node in extracted set where to
        // grip ?
        foreach($branch as $aBranchNode){
            if (array_key_exists($aBranchNode, $items)){
                // grip to that node, initializing childs if necessary
                if (!isset($items[$aBranchNode]->childs)) $items[$aBranchNode]->childs = array();
                $items[$aBranchNode]->childs[] = $item;
                // echo "gripping {$item->id} to {$items[$aBranchNode]->id}<br/>"; 
                $item->fatherid = $aBranchNode;
                // cleanup from root
                unset($items[$item->id]);
                break;
            }
        }        
    }
    
    /**
    * recursively inserts tree structure renumbering items
    * @param int $projectId the project module id 
    * @param int $group the owning group 
    * @param string $table the table-tree where to insert by name
    * @param array $set the set of nodes to insert and make a new tree with
    */
    function insert_tree($projectId, $group, $table, &$set){
        global $CFG;

        // note function is recursive so it has infinite loop protection        
        for($i = 0 ; $i < count($set) && $i < 150; $i++){
            
            // bug fix php4/php5 on reference passing assign
            if (@phpversion() >= 5.0)
                eval(" \$insertedObject = clone \$set[\$i]");
            else
                $insertedObject = $set[$i];

            // get max ordering at root level
            if ($set[$i]->fatherid == 0){
                $position = techproject_tree_get_max_ordering($projectId, $group, $table);
                // remove child as not member in the db data record
                $insertedObject->ordering = $position + 1;
            }

            unset($insertedObject->childs);
            $insertid = insert_record($table, $insertedObject);

            // remap tree distributing inserted id to immediate childs and insert childs
            if (!empty($set[$i]->childs)){
                for($j = 0 ; $j < count($set[$i]->childs)  && $j < 150; $j++){
                    $set[$i]->childs[$j]->fatherid = $insertid;                    
                }
                insert_tree($projectId, $group, $table, $set[$i]->childs);
            }
        }
    }
    
    // first pass, make clones of records in memory.
    $fieldArray = explode(',', $fields);
    foreach($set as $anItem){
        /// get original record
        $node = get_record($fromtable, 'id', $anItem);

        unset($aClone);
        $aClone->id = $node->id;
        $aClone->ordering = $node->ordering;
        $aClone->fatherid = 0; // destroys all tree information as being reconstructed later
        $aClone->lastuserid = $USER->id;
        foreach($fieldArray as $aField){
            $aClone->$aField = mysql_real_escape_string($node->$aField);
        }
        $items[$node->id] = $aClone;
    }
    
   // set project and group context using last viewed node
   $currentprojectId = $node->projectid;
   $currentGroupId = $node->groupid;
    
    // remaps tree structure of collected items
    foreach(array_keys($items) as $anItemId){
        place_in_tree($fromtable, $items, $items[$anItemId]);
    }

    insert_tree($currentprojectId, $currentGroupId, $totable, array_values($items));    
}

/**
* get the max ordering available in sequence at a specified node
* @param project the current project object
* @param group the current group
* @param table the table-tree where to search
* @param istree true id the entity is table-tree rather than table-list
* @param fatherid the parent node
* @return integer the max ordering found
*/
function techproject_tree_get_max_ordering($projectId, $group, $table, $istree = false, $fatherid = 0){
    global $CFG;

    $treeclause = ($istree) ? "AND fatherid = {$fatherid}" : '';
	$query = "
	    SELECT 
	        MAX(ordering) as position
	    FROM 
	        {$CFG->prefix}$table
	    WHERE 
	        groupid = {$group} AND 
	        projectid = {$projectId} 
	        {$treeclause}
	";

	if(! $result = get_record_sql($query)){
		$result->position = 0;
	}
	return $result->position;
}

/**
* get records in tree order
* @param string $table entity-table name
* @param int $projectid the current project
* @param int $groupid the curent group the records belong to
* @param array reference $tree the record tree as an array
* @param int $fatherid whose father we get records for
*/
function techproject_tree_get_tree($table, $projectid, $groupid, &$tree=null, $fatherid=0){
    global $CFG;
    static $deepness = 0;
    static $nodecode;
    
    if (!$tree) $tree = array();
    $sql = "
       SELECT 
          *
       FROM
          {$CFG->prefix}{$table}
       WHERE
          projectid = {$projectid} AND
          groupid = {$groupid} AND
          fatherid = {$fatherid}
       ORDER BY
          ordering
    ";
    $records = get_records_sql($sql);
    if ($records){
        foreach($records as $key => $record){
            $record->deepness = $deepness;
            $record->nodecode = (empty($nodecode)) ? $record->ordering : "$nodecode.{$record->ordering}";
            $tree[$key] = $record;
            $deepness++;
            $oldnode = $nodecode;
            $nodecode = $record->nodecode;
            techproject_tree_get_tree($table, $projectid, $groupid, $tree, $record->id);
            $nodecode = $oldnode;
            $deepness--;
        }
    }
}

/**
* get records in list order (for non tree entities)
* @param string $table entity-table name
* @param int $projectid the current project
* @param int $groupid the curent group the records belong to
* @param array reference $list the record list as an array
*/
function techproject_tree_get_list($table, $projectid, $groupid, &$list=null){
    global $CFG;
    
    if (!$list) $list = array();
    $sql = "
       SELECT 
          *
       FROM
          {$CFG->prefix}{$table}
       WHERE
          projectid = {$projectid} AND
          groupid = {$groupid}
       ORDER BY
          ordering
    ";
    $records = get_records_sql($sql);
    if ($records){
        foreach($records as $key => $record){
            $record->deepness = 0;
            $record->nodecode = $record->ordering ;
            $list[$key] = $record;
        }
    }
}
